/*****************************************************
Chip type               : ATmega16
Program type            : Application
AVR Core Clock frequency: 10,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*****************************************************/

#include <mega16.h>

#include <delay.h>

#define transmit_on() 	{PORTD.2 = 1;}
#define recieve_on()	{PORTD.2 = 0;}

#define led_red         PORTB.5
#define led_green       PORTB.7
#define heater         	PORTB.0

unsigned char heater_state = 0;

unsigned char overheat = 0;

unsigned char connect_cnt = 5;

unsigned char connect_fail_flag = 0;

unsigned int temp_local, temp_ext, temp_bottom, temp_top, temp_max, temp_min;

unsigned char max_flag = 0;
unsigned char min_flag = 0;


eeprom unsigned int null;

eeprom unsigned char t_high = 59;
eeprom unsigned char t_low = 53;

unsigned char t_high_sram = 59;
unsigned char t_low_sram = 53;

#ifndef RXB8
#define RXB8 1
#endif

#ifndef TXB8
#define TXB8 0
#endif

#ifndef UPE
#define UPE 2
#endif

#ifndef DOR
#define DOR 3
#endif

#ifndef FE
#define FE 4
#endif

#ifndef UDRE
#define UDRE 5
#endif

#ifndef RXC
#define RXC 7
#endif

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<DOR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

// USART Receiver buffer
#define RX_BUFFER_SIZE 8
char rx_buffer[RX_BUFFER_SIZE];

#if RX_BUFFER_SIZE <= 256
unsigned char rx_wr_index,rx_rd_index,rx_counter;
#else
unsigned int rx_wr_index,rx_rd_index,rx_counter;
#endif

// This flag is set on USART Receiver buffer overflow
bit rx_buffer_overflow;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
char status,data;
status=UCSRA;
data=UDR;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {
   rx_buffer[rx_wr_index++]=data;
#if RX_BUFFER_SIZE == 256
   // special case for receiver buffer size=256
   if (++rx_counter == 0)
      {
#else
   if (rx_wr_index == RX_BUFFER_SIZE) rx_wr_index=0;
   if (++rx_counter == RX_BUFFER_SIZE)
      {
      rx_counter=0;
#endif
      rx_buffer_overflow=1;
      }
   }
}

#ifndef _DEBUG_TERMINAL_IO_
// Get a character from the USART Receiver buffer
#define _ALTERNATE_GETCHAR_
#pragma used+
char getchar(void)
{
char data;
while (rx_counter==0);
data=rx_buffer[rx_rd_index++];
#if RX_BUFFER_SIZE != 256
if (rx_rd_index == RX_BUFFER_SIZE) rx_rd_index=0;
#endif
#asm("cli")
--rx_counter;
#asm("sei")
return data;
}
#pragma used-
#endif

// Standard Input/Output functions
#include <stdio.h>

#define ADC_VREF_TYPE 0xC0

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
// Delay needed for the stabilization of the ADC input voltage
delay_us(10);
// Start the AD conversion
ADCSRA|=0x40;
// Wait for the AD conversion to complete
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCW;
}

unsigned int ms5_cnt = 0;	//   5 
unsigned int s1_cnt = 0;       //   1

unsigned char led_red_cnt;
unsigned char led_red_pwm;
unsigned char red_up = 0;

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
        {
        TCNT0=0xC1;	//  50 
        #asm("wdr")
        
        if (++ms5_cnt == 100)
        	{
                ms5_cnt = 0;
                 
                if (!heater_state && !overheat)                              
                        {
                        if (red_up) 
                                { 
                                if (led_red_cnt<255) led_red_cnt++; 
                                        else red_up = 0; 
                                }
                                else
                                        {
                                        if (led_red_cnt>0) led_red_cnt--; 
                                                else red_up = 1;
                                        } 
                        };
                }
                
        if (++s1_cnt == 5000)
        	{
                s1_cnt = 0;
                if (overheat) led_red = !led_red;
                if (connect_fail_flag) led_green = !led_green;
                if (connect_cnt) connect_cnt--;
                	else connect_fail_flag = 1;
                }
                
   	led_red_pwm++;
        if (!overheat)
        	{
                if (!heater_state)
                        {
                        if (led_red_cnt > led_red_pwm) led_red = 1; else led_red = 0;
                        }
                        else led_red = 1;
                }
        }

void get_top_temp (void)
	{
        temp_top = (read_adc(2)+read_adc(2))*5/4; 
        if (temp_top%10 >= 5) temp_top = temp_top/10+1;
        	else temp_top = temp_top/10; 
        }
        
void get_bottom_temp (void)
	{
        temp_bottom = (read_adc(1)+read_adc(1))*5/4; 
        if (temp_bottom%10 >= 5) temp_bottom = temp_bottom/10+1;
        	else temp_bottom = temp_bottom/10; 
        }
        
void get_ext_temp (void)
	{
        temp_ext = (read_adc(0)+read_adc(0))*5/4; 
        if (temp_ext%10 >= 5) temp_ext = temp_ext/10+1;
        	else temp_ext = temp_ext/10; 
        }

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTA=0x00;
DDRA=0x00;

// Port B initialization
// Func7=Out Func6=Out Func5=Out Func4=In Func3=In Func2=In Func1=In Func0=Out 
// State7=0 State6=0 State5=0 State4=T State3=T State2=T State1=T State0=0 
PORTB=0x00;
DDRB=0xE1;

// Port C initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=Out Func1=Out Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=0 State1=0 State0=P 
PORTD=0x01;
DDRD=0x06;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 1,250,000 kHz
// OC0 output: Disconnected
TCCR0=0x02;
TCNT0=0xC1;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer1 Stopped
// Mode: Normal top=0xFFFF
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer2 Stopped
// Mode: Normal top=0xFF
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
MCUCR=0x00;
MCUCSR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 25000
UCSRA=0x00;
UCSRB=0x98;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x18;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 625,000 kHz
// ADC Voltage Reference: Int., cap. on AREF
// ADC Auto Trigger Source: ADC Stopped
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x84;

// SPI initialization
// SPI disabled
SPCR=0x00;

// TWI initialization
// TWI disabled
TWCR=0x00;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/2048k
#pragma optsize-
WDTCR=0x1F;
WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

if (null > 1)
	{
        null = 0;
        t_high = 59;
	t_low = 53;
	}
        
t_high_sram = t_high;
t_low_sram = t_low;

// Global enable interrupts
#asm("sei")

while (1)
      	{
      	recieve_on();
        delay_ms(5);
        get_ext_temp ();
      	get_top_temp ();
      	get_bottom_temp ();
        
        if (temp_top > temp_bottom) temp_max = temp_top;
        	else temp_max = temp_bottom;
                          
        if (rx_counter)
                {
                if (getchar() == 0xAA)
                        {
                        led_green = 1;
                        delay_ms(50);
                        if (getchar() == 0xBB)
                                {
                                if (getchar() == 0xCC)
                                        {
                                        if (temp_max <= 70) heater_state = getchar()-0xDD;
                                        	else heater_state = 0; 
                                        t_low_sram = getchar();
                                        t_high_sram = getchar();
                                        
                                        if (t_low != t_low_sram) t_low = t_low_sram;
                                        if (t_high != t_high_sram) t_high = t_high_sram;
                                        
                                        connect_cnt = 5;
                                        connect_fail_flag = 0;
                                        
                                        led_green = 0;
                                        
                                        delay_ms(100);
                                        
                                        transmit_on(); 
                                        
                                        putchar(0xAA);
                                        putchar(0xBB);
                                        putchar(temp_ext);
                                        putchar(temp_top);
                                        putchar(temp_bottom);
                                        putchar(heater_state);
                                        delay_ms(10);
                                        }
                                } 
                        }
                }
                
        if (connect_fail_flag)
        	{
        	if (temp_top<=temp_bottom) temp_min = temp_top;
                	else temp_min = temp_bottom;
                
                if (temp_min == t_low) {min_flag = 1; max_flag = 0; heater_state = 1;};
                if (temp_min == t_high) {min_flag = 0; max_flag = 1; heater_state = 0;};

                if (temp_min < t_high && temp_min > t_low)
                        {
                        if (min_flag && !max_flag) heater_state = 1;
                        if (!min_flag && max_flag) heater_state = 0; 
                        } 

                if (temp_min < t_low) heater_state = 1;
                if (temp_min > t_high) heater_state = 0;
                }
        
        if (temp_max <= 70) {heater = heater_state; overheat = 0;}
        	else {heater_state = 0; heater = 0; overheat = 1;}

      	}
}
