/*****************************************************
Chip type               : ATmega8
Program type            : Application
AVR Core Clock frequency: 10,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*****************************************************/

#include <mega8.h>

#include <delay.h>

#define buzzer_on() 	{PORTD.3 = 1;}
#define buzzer_off()	{PORTD.3 = 0;}

#define transmit_on() 	{PORTB.1 = 1;}
#define recieve_on()	{PORTB.1 = 0;}

#define but1		!PIND.7
#define but2		!PIND.6
#define but3		!PIND.5

#define SPI_PORT        PORTB
#define SPI_DDR         DDRB

#define SS_PIN          2
#define MOSI_PIN        3
#define MISO_PIN        4
#define SCK_PIN         5 
                  
#define strobe_PORT     PORTB
#define strobe_DDR      DDRB
#define strobe_PIN      0
#define strobe_toggle() {strobe_PORT.strobe_PIN = 1; strobe_PORT.strobe_PIN = 0;}

unsigned int temp_local, temp_ext, temp_bottom, temp_top, temp_min;

unsigned char max_flag = 0;
unsigned char min_flag = 0;

unsigned char heater_state = 0;
unsigned char power_heater_state = 0;	//     

unsigned char connect_cnt = 5;

unsigned char connect_fail_flag = 0;

unsigned char show_cnt, setup_cnt = 0;

unsigned char byte1, byte0;

unsigned char heater_always_on = 0;

#define led1_on()	{byte1 |= 0b00010000;}
#define led1_off()	{byte1 &= 0b11101111;}
#define led2_on()	{byte1 |= 0b00100000;}
#define led2_off()	{byte1 &= 0b11011111;}
#define led3_on()	{byte1 |= 0b01000000;}
#define led3_off()	{byte1 &= 0b10111111;}

eeprom unsigned int null;

eeprom unsigned char t_high = 59;
eeprom unsigned char t_low = 53;

#ifndef RXB8
#define RXB8 1
#endif

#ifndef TXB8
#define TXB8 0
#endif

#ifndef UPE
#define UPE 2
#endif

#ifndef DOR
#define DOR 3
#endif

#ifndef FE
#define FE 4
#endif

#ifndef UDRE
#define UDRE 5
#endif

#ifndef RXC
#define RXC 7
#endif

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<DOR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

// USART Receiver buffer
#define RX_BUFFER_SIZE 8
char rx_buffer[RX_BUFFER_SIZE];

#if RX_BUFFER_SIZE <= 256
unsigned char rx_wr_index,rx_rd_index,rx_counter;
#else
unsigned int rx_wr_index,rx_rd_index,rx_counter;
#endif

// This flag is set on USART Receiver buffer overflow
bit rx_buffer_overflow;

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
char status,data;
status=UCSRA;
data=UDR;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {
   rx_buffer[rx_wr_index++]=data;
#if RX_BUFFER_SIZE == 256
   // special case for receiver buffer size=256
   if (++rx_counter == 0)
      {
#else
   if (rx_wr_index == RX_BUFFER_SIZE) rx_wr_index=0;
   if (++rx_counter == RX_BUFFER_SIZE)
      {
      rx_counter=0;
#endif
      rx_buffer_overflow=1;
      }
   }
}

#ifndef _DEBUG_TERMINAL_IO_
// Get a character from the USART Receiver buffer
#define _ALTERNATE_GETCHAR_
#pragma used+
char getchar(void)
{
char data;
while (rx_counter==0);
data=rx_buffer[rx_rd_index++];
#if RX_BUFFER_SIZE != 256
if (rx_rd_index == RX_BUFFER_SIZE) rx_rd_index=0;
#endif
#asm("cli")
--rx_counter;
#asm("sei")
return data;
}
#pragma used-
#endif

// Standard Input/Output functions
#include <stdio.h>

// I2C Bus functions
#asm
   .equ __i2c_port=0x15 ;PORTC
   .equ __sda_bit=4
   .equ __scl_bit=5
#endasm
#include <i2c.h>

// DS1307 Real Time Clock functions
#include <ds1307.c>

void sound_on (int time)	//  
        {
        int i = 0;
        while (i < time*2)
                {  
                buzzer_on();
                delay_us(250);
                buzzer_off();
                delay_us(250);                
                i++;
                }                                                
        }

void SPIInit(void)
        {
        unsigned char temp;     // 
        
                                // setup SPI I/O pins
        SPI_DDR.SCK_PIN = 1;    // set SCK as output
        SPI_PORT.SCK_PIN = 0;   // set SCK lo
        
        SPI_DDR.MISO_PIN = 0;   // set MISO as input
        SPI_PORT.MISO_PIN = 1;  // set pullup
        
        SPI_DDR.MOSI_PIN = 1;   // set MOSI as output
        
        SPI_DDR.SS_PIN = 1;     // SS must be output for Master mode to work
        
        strobe_DDR.strobe_PIN = 1;      //     
        strobe_PORT.strobe_PIN = 0;     //  
        
                                //SPI Mode: 0
	                                
        SPCR.0 = 0;             //SPI Clock Rate Select (fosc/4)
	SPCR.1 = 0;             //
	SPSR.0 = 1;             //SPI2X: Double SPI Speed (1) Bit, ..    fosc/2
	SPCR.2 = 0;             //CPHA: Clock Phase (  - 1)- . 
	SPCR.3 = 0;             //CPOL: Clock Polarity (0:   - ,   - ) - . 
	SPCR.4 = 1;             //MSTR: Master(1)/Slave(0) Select
	SPCR.5 = 0;             //DORD: Data Order (LSB first = 1; MSB first = 0)
	SPCR.6 = 1;             //SPE: SPI Enable = 1
	SPCR.7 = 0;             //SPIE: SPI Interrupt Enable = 1
	
	temp = SPSR;            //clear SPI status (. )
	temp = SPDR;
	
	SPDR = 0; while (!SPSR.7) #asm("wdr");	//  0
        SPDR = 0; while (!SPSR.7) #asm("wdr");	//  0
	strobe_toggle();        // Strobe 
        };

unsigned char ValueToSegment(unsigned char val)
        {
        unsigned char res=0;
        switch(val)
                {
                case 0: res=0b01111110;	break; 
      		case 1: res=0b00000110;	break; 
      		case 2: res=0b01110101;	break; 
                case 3: res=0b01010111;	break; 
                case 4: res=0b00001111;	break; 
                case 5: res=0b01011011;	break; 
                case 6: res=0b01111011;	break; 
                case 7: res=0b00010110;	break; 
                case 8: res=0b01111111;	break; 
                case 9: res=0b01011111;	break; 
                case 'C': res=0b01111000;	break; 
                case '*': res=0b00011101;	break; 
                case '-': res=0b00000001;	break; 
                default: res=0; // ,   
                }
        return res;
        }
        
unsigned char screen_buf[4];
unsigned char display_timer_cnt = 0;

unsigned int button_cnt = 0;
unsigned char long_press = 0;

unsigned int s1_cnt = 0;       //   1
unsigned char second_flag = 0;

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
	{
        //TCNT0=0xC1;
        #asm("wdr")
        
        if (++display_timer_cnt == 4) display_timer_cnt = 0;
	byte0 = screen_buf[display_timer_cnt];
	switch (display_timer_cnt)
		{
		case 0: {
			byte1 &= 0b11110000;
                        byte1 |= 0b00001000;
                        }; break;
		case 1: {
			byte1 &= 0b11110000;
                        byte1 |= 0b00000100;
                        }; break;
		case 2: {
			byte1 &= 0b11110000;
                        byte1 |= 0b00000010;
                        }; break;
		case 3: {
			byte1 &= 0b11110000;
                        byte1 |= 0b00000001;
                        };
		}
        SPDR = byte1;            	//  
        while (!SPSR.7) #asm ("wdr");   //,    
        SPDR = byte0;            	//  
        while (!SPSR.7) #asm ("wdr");   //,    
        strobe_toggle();        	// Strobe         
        
        if (++s1_cnt == 5000)
        	{
                s1_cnt = 0;
                second_flag = 1;
                if (setup_cnt) setup_cnt--;
                }
                
        if (but3)
        	{
                if (++button_cnt == 5000)
                	{
                        button_cnt = 0;
                        if (long_press < 3) long_press++;
                        } 
                }
                else button_cnt = 0; 
        }
        
void print2screenbuf (unsigned char s1, unsigned char s2, unsigned char s3, unsigned char s4, unsigned char p1, unsigned char p2,unsigned char p3,unsigned char p4, unsigned int time)  
	{
        screen_buf[0] = ValueToSegment(s1);
        screen_buf[1] = ValueToSegment(s2);
        screen_buf[2] = ValueToSegment(s3);
        screen_buf[3] = ValueToSegment(s4);
        
        if (p1) screen_buf[0] |= 0b10000000;
        if (p2) screen_buf[1] |= 0b10000000;
        if (p3) screen_buf[2] |= 0b10000000;
        if (p4) screen_buf[3] |= 0b10000000;
        
        if (heater_always_on) screen_buf[3] |= 0b10000000; 
        
        delay_ms(time);
        }

#define ADC_VREF_TYPE 0xC0

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
// Delay needed for the stabilization of the ADC input voltage
delay_us(10);
// Start the AD conversion
ADCSRA|=0x40;
// Wait for the AD conversion to complete
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCW;
}

void get_local_temp (void)
	{
        temp_local = (read_adc(7)+read_adc(7))*5/4;
        if (temp_local%10 >= 5) temp_local = temp_local/10+1;
        	else temp_local = temp_local/10; 
        }
        
void Setup_time (void)
	{
        unsigned char setup_index = 0;
        unsigned char ch_flag1 = 0;
        unsigned char ch_flag2 = 0;
        
        setup_cnt = 15;
        print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,0);
        
        while (setup_cnt)
        	{
                if (setup_index == 0)
                	{
                	if (!but1 && !but2 && !but3) print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,500);
                	if (!but1 && !but2 && !but3) print2screenbuf (66, 66, time [1]/10, time [1]%10, 0,1,0,0,500);
                        if (but2)
                        	{
                                delay_ms(10);
                                while (but2)
                                        {
                                        ch_flag1 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        if (++time[0] > 23) time[0] = 0;        
                                        print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,300);
                                        }
                                }
                                
                        if (but1)
                        	{
                                delay_ms(10);
                                while (but1)
                                        {
                                        ch_flag1 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        if (--time[0] > 23) time[0] = 23;        
                                        print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,300);
                                        }
                                }
                        }
                
                if (setup_index == 1)
                	{
                	print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,500);
                	print2screenbuf (time [0]/10, time [0]%10, 66, 66, 0,1,0,0,500);
                        if (but2)
                        	{
                                delay_ms(10);
                                while (but2)
                                        {
                                        ch_flag2 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        if (++time[1] > 59) time[1] = 0;        
                                        print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,300);
                                        }
                                }
                                
                        if (but1)
                        	{
                                delay_ms(10);
                                while (but1)
                                        {
                                        ch_flag2 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        if (--time[1] > 59) time[1] = 59;        
                                        print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,300);
                                        }
                                }
                        }
                
                if (but3)
                	{
                        delay_ms(10);
                        if (but3)
                        	{
                                if (setup_index == 1)
                                	{
                                        setup_cnt = 0;
                                        if (ch_flag2) rtc_set_time(time [0], time [1], time [2]);
                                        sound_on(50);
                                        }
                                if (setup_index == 0)
                                	{
                                        setup_cnt = 15;
                                        setup_index = 1;
                                        if (ch_flag1) rtc_set_time(time [0], time [1], time [2]);
                                        sound_on(50);
                                        }
                                while (but3);
                                }
                        }
                } 
        }
        
void Setup_temp (void)
	{
        unsigned char setup_index = 0;
        unsigned char ch_flag1 = 0;
        unsigned char ch_flag2 = 0;
        unsigned char local_lo, local_hi;
        
        local_lo = t_low;
        local_hi = t_high;
        setup_cnt = 15;
        print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,0);
        
        while (setup_cnt)
        	{
                if (setup_index == 0)
                	{
                	if (!but1 && !but2 && !but3) print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,500);
                	if (!but1 && !but2 && !but3) print2screenbuf (66, '*', (local_hi-50), '*', 0,0,0,0,500);
                        if (but2)
                        	{
                                delay_ms(10);
                                while (but2)
                                        {
                                        ch_flag1 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        local_lo++;
                                        if (local_lo > (local_hi-3)) local_lo = 53;        
                                        print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,300);
                                        }
                                }
                                
                        if (but1)
                        	{
                                delay_ms(10);
                                while (but1)
                                        {
                                        ch_flag1 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        local_lo--;
                                        if (local_lo < 53) local_lo = local_hi-3;        
                                        print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,300);
                                        }
                                }
                        }
                
                if (setup_index == 1)
                	{
                	if (!but1 && !but2 && !but3) print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,500);
                	if (!but1 && !but2 && !but3) print2screenbuf ((local_lo-50), '*', 66, '*', 0,0,0,0,500);
                        if (but2)
                        	{
                                delay_ms(10);
                                while (but2)
                                        {
                                        ch_flag2 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        local_hi++;
                                        if (local_hi > 59) local_hi = local_lo + 3;        
                                        print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,300);
                                        }
                                }
                                
                        if (but1)
                        	{
                                delay_ms(10);
                                while (but1)
                                        {
                                        ch_flag2 = 1;
                                        setup_cnt = 15;
                                        sound_on(50);
                                        local_hi--;
                                        if (local_hi < (local_lo + 3)) local_hi = 59;        
                                        print2screenbuf ((local_lo-50), '*', (local_hi-50), '*', 0,0,0,0,300);
                                        }
                                }
                        }
                
                if (but3)
                	{
                        delay_ms(10);
                        if (but3)
                        	{
                                if (setup_index == 1)
                                	{
                                        setup_cnt = 0;
                                        if (ch_flag2) t_high = local_hi;
                                        sound_on(50);
                                        }
                                if (setup_index == 0)
                                	{
                                        setup_cnt = 15;
                                        setup_index = 1;
                                        if (ch_flag1) t_low = local_lo;
                                        sound_on(50);
                                        }
                                while (but3);
                                }
                        }
                } 
        }

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=Out Func0=Out 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=0 State0=0 
PORTB=0x00;
DDRB=0x03;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=Out Func2=In Func1=Out Func0=In 
// State7=P State6=P State5=P State4=T State3=0 State2=T State1=0 State0=P 
PORTD=0xE1;
DDRD=0x0A;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 1000,000 kHz
TCCR0=0x02;
//TCNT0=0xC1;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer1 Stopped
// Mode: Normal top=0xFFFF
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer2 Stopped
// Mode: Normal top=0xFF
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 25000
UCSRA=0x00;
UCSRB=0x98;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x18;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 625,000 kHz
// ADC Voltage Reference: Int., cap. on AREF
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x84;

SPIInit();

// TWI initialization
// TWI disabled
TWCR=0x00;

// I2C Bus initialization
i2c_init();

// DS1307 Real Time Clock initialization
// Square wave output on pin SQW/OUT: Off
// SQW/OUT pin state: 0
rtc_init(0,0,0);

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/2048k
#pragma optsize-
WDTCR=0x1F;
WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")

if (null > 1)
	{
        null = 0;
        t_high = 59;
	t_low = 53;
	}

print2screenbuf (3, 3, 3, 3, 0,0,0,0,1000);
print2screenbuf (2, 2, 2, 2, 0,0,0,0,1000);

get_local_temp ();
rtc_get_time ();

print2screenbuf (1, 1, 1, 1, 0,0,0,0,1000);

led1_on();
sound_on(200);

while (1)
      	{
      	if (show_cnt == 0) print2screenbuf (time [0]/10, time [0]%10, time [1]/10, time [1]%10, 0,1,0,0,0); 
      	if (show_cnt == 2 && !connect_fail_flag)
                {
                if ((temp_ext>40) && (temp_ext<60)) 
                        {
                        if (temp_ext<50) print2screenbuf (66, '-', (50-temp_ext), '*', 0,0,0,0,0);
                                else print2screenbuf ('66', (temp_ext-50), '*', 66, 0,0,0,0,0);
                        }
                        else
                                {
                                if (temp_ext<50) print2screenbuf ('-', (50-temp_ext)/10, (50-temp_ext)%10, '*', 0,0,0,0,0);
                                        else print2screenbuf ('66', (temp_ext-50)/10, (temp_ext-50)%10, '*', 0,0,0,0,0);
                                }
                }
        
      
      
      	if (but1 || but2 || but3)
        	{
                delay_ms(5);
                if (but1) 
                	{
                        sound_on(50);
                        second_flag = 0;
                        if ((temp_bottom>40) && (temp_bottom<60)) 
                                {
                                if (temp_bottom<50) print2screenbuf (66, '-', (50-temp_bottom), '*', 0,0,0,0,2000);
                                        else print2screenbuf ('66', (temp_bottom-50), '*', 66, 0,0,0,0,2000);
                                }
                                else
                                        {
                                        if (temp_bottom<50) print2screenbuf ('-', (50-temp_bottom)/10, (50-temp_bottom)%10, '*', 0,0,0,0,2000);
                                                else print2screenbuf ('66', (temp_bottom-50)/10, (temp_bottom-50)%10, '*', 0,0,0,0,2000);
                                        }
                        }
                if (but2)
                	{
                        sound_on(50);
                        second_flag = 0;
                        if ((temp_top>40) && (temp_top<60)) 
                                {
                                if (temp_top<50) print2screenbuf (66, '-', (50-temp_top), '*', 0,0,0,0,2000);
                                        else print2screenbuf ('66', (temp_top-50), '*', 66, 0,0,0,0,2000);
                                }
                                else
                                        {
                                        if (temp_top<50) print2screenbuf ('-', (50-temp_top)/10, (50-temp_top)%10, '*', 0,0,0,0,2000);
                                                else print2screenbuf ('66', (temp_top-50)/10, (temp_top-50)%10, '*', 0,0,0,0,2000);
                                        }
                        }
                if (but3)
                	{
                        sound_on(50);
                        while (but3 && long_press == 0);
                        if (but3 && long_press == 1) 
                        	{
                                sound_on(100);
                                }
                        while (but3 && long_press == 1);
                        if (but3 && long_press == 2) 
                        	{
                                sound_on(100);
                                delay_ms(100);
                                sound_on(100);
                                }
                        while (but3 && long_press == 2);
                        if (but3 && long_press == 3) 
                        	{
                                sound_on(100);
                                delay_ms(100);
                                sound_on(100);
                                delay_ms(100);
                                sound_on(100);
                                }
                        while (but3);
                        if (long_press == 1) {long_press = 0; Setup_time();}
                        	else if (long_press == 2) {long_press = 0; Setup_temp();}
                                	else if (long_press == 3) 
                                        	{
                                                long_press = 0;
                                                if (heater_always_on) heater_always_on = 0;
                                                	else heater_always_on = 1;
                                                }
                                                else 
                                                        {
                                                        long_press = 0;
                                                        second_flag = 0;
                                                        if ((temp_local>40) && (temp_local<60)) 
                                                                {
                                                                if (temp_local<50) print2screenbuf (66, '-', (50-temp_local), '*', 0,0,0,0,2000);
                                                                        else print2screenbuf ('66', (temp_local-50), '*', 66, 0,0,0,0,2000);
                                                                }
                                                                else
                                                                        {
                                                                        if (temp_local<50) print2screenbuf ('-', (50-temp_local)/10, (50-temp_local)%10, '*', 0,0,0,0,2000);
                                                                                else print2screenbuf ('66', (temp_local-50)/10, (temp_local-50)%10, '*', 0,0,0,0,2000);
                                                                        }
                                                        }
                        
                        }
                while (but2 || but1);
                }
      
      
      	if (second_flag)
        	{
                second_flag = 0;
                transmit_on();
                delay_ms(5);
                
                if (++show_cnt == 4) show_cnt = 0;
                
                get_local_temp ();
                
                if (connect_fail_flag)
                	{
                        if ((byte1&0b00100000) == 0b00100000) led2_off()
                        	else led2_on();
                        }
                
                if (temp_top<=temp_bottom) temp_min = temp_top;
                	else temp_min = temp_bottom;
                
                if (temp_min == t_low) {min_flag = 1; max_flag = 0; heater_state = 1;};
                if (temp_min == t_high) {min_flag = 0; max_flag = 1; heater_state = 0;};

                if (temp_min < t_high && temp_min > t_low)
                        {
                        if (min_flag && !max_flag) heater_state = 1;
                        if (!min_flag && max_flag) heater_state = 0; 
                        } 

                if (temp_min < t_low) heater_state = 1;
                if (temp_min > t_high) heater_state = 0;
                
                if (heater_always_on) heater_state = 1;
                
                putchar(0xAA);
                delay_us(100);
                putchar(0xBB);
                delay_us(100);
                putchar(0xCC);
                delay_us(100);
                putchar(0xDD+heater_state);
                delay_us(100);
                putchar(t_low);
                delay_us(100);
                putchar(t_high);
                
                if (connect_cnt) connect_cnt--;
                	else 
                        	{
                                connect_fail_flag = 1; 
                                power_heater_state = 0;
                                }
                delay_ms(5);
                recieve_on();
                rtc_get_time();
                delay_ms(5);
                if (!connect_fail_flag) led2_off();
                }
                
        if (rx_counter)
        	{
                if (getchar() == 0xAA)
                	{
                        led2_on();
                        delay_ms(50);
                        if (getchar() == 0xBB)
                        	{
                                temp_ext = getchar();
                                temp_top = getchar();
                                temp_bottom = getchar();
                                power_heater_state = getchar();
                                connect_cnt = 5;
                                connect_fail_flag = 0;
                                } 
                        led2_off();
                        }
                }
      	
        if (power_heater_state && !connect_fail_flag) led3_on()
        	else led3_off();
        }
}
